/* AVR-Demo for MenuInterpreter
  Version 1.0
  (c) 2009 by Malte Marwedel
  www.marwedels.de/malte

  This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


possible md5sum of the hex file while compiling for an ATMega8:
6946847d10f68bd0465b12275dfe3b7b
program size: 6722 byte


BEFORE COMPILING, DONT FORGET TO ADD 'PROGMEM' INTO menudata.c!

*/

#include <avr/io.h>
#include <avr/pgmspace.h>

#include "../menu-interpreter.h"


#include "menudata.c"

unsigned char screen[MENU_SCREEN_Y][MENU_SCREEN_X/8+1];

/* uart functions */

#define F_CPU 8000000
#define baudrate 38400
#define uartnumber (F_CPU/(baudrate*16l)-1)

uint8_t uart_get(void) {
	if (bit_is_set(UCSRA,RXC)) {
		return UDR;
	}else{
		return 0;
	}
}

void uart_put(uint8_t value ) {
	loop_until_bit_is_set(UCSRA,5);
	UDR = value;
}

void uart_init(void) {
	UBRRL = uartnumber; //speed
	UCSRB = 0x18;      //Transmitter and Reciver enabled
}

/* Functions for the lib */

unsigned char menu_byte_get(MENUADDR addr) {
	if (addr >= MENU_DATASIZE) {
		return 0;
	}
	return pgm_read_byte(menudata+addr);
}

void menu_screen_set(unsigned char x, unsigned char y, unsigned char color) {
	if ((x < MENU_SCREEN_X) && (y < MENU_SCREEN_Y)) {
		unsigned char b = screen[y][x/8];
		if (color & 1) {
			b |= (1<< (x % 8));
		} else
			b &= ~(1<<(x % 8));
		screen[y][x/8] = b;
	}
}

void menu_screen_flush(void) {
	unsigned char i, j;
	for (i = 0; i < MENU_SCREEN_Y; i++) {
		j = 0;
		while (j < MENU_SCREEN_X) {
			unsigned char b = screen[i][j/8];
			unsigned char k;
			for (k = 0; k < 8; k++) {
				unsigned char c = 0;
				if (b & (1<<k)) {
					c = 1;
				}
				if (menu_cheboxstate[MENU_CHECKBOX_invert]) //invert
					c = 1-c;
				if (c) {
					c = '#';
					if (menu_radiobuttonstate[MENU_RBUTTON_radio1] == 1)
						c = '*';
					if (menu_radiobuttonstate[MENU_RBUTTON_radio1] == 2)
						c = 'x';
				} else
					c = ' ';
				uart_put(c);
				j++;
			}
			if (j >= MENU_SCREEN_X)
				break;
		}
		uart_put('\r');
		uart_put('\n');
	}
		uart_put('\r');
	uart_put('\n');
}

void menu_screen_clear(void) {
	unsigned char i, j;
	for (i = 0; i < MENU_SCREEN_Y; i++) {
		for (j = 0; j < (MENU_SCREEN_X/8+1); j++) {
			screen[i][j] = 0;
		}
	}
}

unsigned char listtext[] = "Select: 0,0\0";

unsigned char pintext[] = "=00000000\0";

unsigned char samples = 0;
unsigned char oldportstate = 0;

void menu_action(unsigned short action) {
	if (action == MENU_ACTION_LIST1) {
		listtext[8] = 48+menu_listindexstate[MENU_LISTINDEX_list1];
		listtext[10] = 48+menu_listindexstate[MENU_LISTINDEX_list2];
		menu_redraw();
	}
	if (action == MENU_ACTION_SAMPLESTART)
		samples = 1;
	if (action == MENU_ACTION_SAMPLESTOP)
		samples = 0;
}

void portsample(void) {
	if (samples) {
		unsigned char state = PINB;
		if (state != oldportstate) {
			oldportstate = state;
			unsigned char i;
			for (i = 0; i < 8; i++) {
				if (state & 0x80) {
					pintext[1+i] = '1';
				} else
					pintext[1+i] = '0';
				state = state <<1;
			}
			menu_redraw();
		}
	}
}

int main(void) {
	uart_init();
	uart_put('S');
	uart_put('t');
	uart_put('a');
	uart_put('r');
	uart_put('t');
	uart_put('\r');
	uart_put('\n');
	menu_strings[MENU_TEXT_LISTSEL] = listtext;
	menu_strings[MENU_TEXT_PORTB] = pintext;
	menu_redraw();
	while(1) {
		unsigned char key = uart_get();
		if (key) {
			if (menu_cheboxstate[MENU_CHECKBOX_echo]) {
				uart_put(key);
				uart_put('\r');
				uart_put('\n');
			}
			menu_keypress(key);
		}
		portsample();
	}
	return 0;
}
