/* PC-Demo for MenuInterpreter
  Version 1.0
  (c) 2009 by Malte Marwedel
  www.marwedels.de/malte

  This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>


#include "../menu-interpreter.h"


#include "menudata.c"

unsigned char screen[MENU_SCREEN_Y][MENU_SCREEN_X/8+1];

/* Functions for the lib */

unsigned char menu_byte_get(MENUADDR addr) {
	if (addr >= MENU_DATASIZE) {
		printf("Error: %li out of range\n", (long int)addr);
		//force a segfault for easy backtracing
		//return menudata[0x787654321];
		exit(1);
	}
	return menudata[addr];
}

void menu_screen_set(unsigned char x, unsigned char y, unsigned char color) {
	if ((x < MENU_SCREEN_X) && (y < MENU_SCREEN_Y)) {
		unsigned char b = screen[y][x/8];
		if (color & 1) {
			b |= (1<< (x % 8));
		} else
			b &= ~(1 << (x % 8));
		screen[y][x/8] = b;
	} else {
#ifdef DEBUG
		printf("Warning: Drawing %i, %i is out of bounds\n", x, y);
#endif
	}
}

void menu_screen_flush(void) {
	//system("clear");
	int i, j;
	for (i = 0; i < MENU_SCREEN_Y; i++) {
		j = 0;
		while (j < MENU_SCREEN_X) {
			unsigned char b = screen[i][j/8];
			unsigned char k;
			for (k = 0; k < 8; k++) {
				if (b & (1<<k)) {
					printf("#");
				} else
					printf(" ");
				j++;
			}
			if (j >= MENU_SCREEN_X)
				break;
		}
		printf("\n");
	}
	printf("\n");
}

void menu_screen_clear(void) {
	int i, j;
	for (i = 0; i < MENU_SCREEN_Y; i++) {
		for (j = 0; j < (MENU_SCREEN_X/8+1); j++) {
			screen[i][j] = 0;
		}
	}
}

void menu_action(unsigned short action) {
	printf("Event %i called\n", action);
}

int main(void) {
	menu_redraw();
	while(1) {
		int key = getchar();
		menu_keypress(key);
	}
	return 0;
}
